@@ -24,11 +24,13 @@ import uuid
 import zipfile
 from contextlib import closing
 from datetime import date, datetime, timezone, tzinfo
+from json import JSONDecodeError
 from multiprocessing.dummy import Pool
 from queue import Queue
 from typing import Any, Callable, Dict, List, Optional, Sized, Tuple, Type, Union
 from urllib.parse import parse_qs, urlparse
 
+import cachetools
 import dns.resolver
 import requests
 import six
@@ -525,6 +527,64 @@ class FileMappedDocument(dict):
             json.dump(self, fd)
 
 
+class PortNotAvailableException(Exception):
+    """Exception which indicates that the ExternalServicePortsManager could not reserve a port."""
+
+    pass
+
+
+class ExternalServicePortsManager:
+    """Manages the ports used for starting external services like ElasticSearch, OpenSearch,..."""
+
+    def __init__(self):
+        # cache for locally available ports (ports are reserved for a short period of a few seconds)
+        self._PORTS_CACHE = cachetools.TTLCache(maxsize=100, ttl=6)
+        self._PORTS_LOCK = threading.RLock()
+
+    def reserve_port(self, port: int = None) -> int:
+        """
+        Reserves the given port (if it is still free). If the given port is None, it reserves a free port from the
+        configured port range for external services. If a port is given, it has to be within the configured
+        range of external services (i.e. in [config#EXTERNAL_SERVICE_PORTS_START, config#EXTERNAL_SERVICE_PORTS_END)).
+        :param port: explicit port to check or None if a random port from the configured range should be selected
+        :return: reserved, free port number (int)
+        :raises: PortNotAvailableException if the given port is outside the configured range, it is already bound or
+                    reserved, or if the given port is none and there is no free port in the configured service range.
+        """
+        ports_range = range(config.EXTERNAL_SERVICE_PORTS_START, config.EXTERNAL_SERVICE_PORTS_END)
+        if port is not None and port not in ports_range:
+            raise PortNotAvailableException(
+                f"The requested port ({port}) is not in the configured external "
+                f"service port range ({ports_range})."
+            )
+        with self._PORTS_LOCK:
+            if port is not None:
+                return self._check_port(port)
+            else:
+                for port_in_range in ports_range:
+                    try:
+                        return self._check_port(port_in_range)
+                    except PortNotAvailableException:
+                        # We ignore the fact that this single port is reserved, we just check the next one
+                        pass
+        raise PortNotAvailableException(
+            "No free network ports available to start service instance (currently reserved: %s)",
+            list(self._PORTS_CACHE.keys()),
+        )
+
+    def _check_port(self, port: int) -> int:
+        """Checks if the given port is currently not reserved and can be bound."""
+        if not self._PORTS_CACHE.get(port) and port_can_be_bound(port):
+            # reserve the port for a short period of time
+            self._PORTS_CACHE[port] = "__reserved__"
+            return port
+        else:
+            raise PortNotAvailableException(f"The given port ({port}) is already reserved.")
+
+
+external_service_ports = ExternalServicePortsManager()
+
+
 # ----------------
 # UTILITY METHODS
 # ----------------
@@ -943,6 +1003,10 @@ def merge_dicts(*dicts, **kwargs):
     return result
 
 
+def remove_none_values_from_dict(dict: Dict) -> Dict:
+    return {k: v for (k, v) in dict.items() if v is not None}
+
+
 def recurse_object(obj: JsonType, func: Callable, path: str = "") -> Any:
     """Recursively apply `func` to `obj` (may be a list, dict, or other object)."""
     obj = func(obj, path=path)
@@ -975,8 +1039,11 @@ def keys_to_lower(obj: JsonComplexType, skip_children_of: List[str] = None) -> J
     return result
 
 
+_camel_to_snake_case_sub = re.compile("((?<=[a-z0-9])[A-Z]|(?!^)[A-Z](?=[a-z]))")
+
+
 def camel_to_snake_case(string: str) -> str:
-    return re.sub(r"(?<!^)(?=[A-Z])", "_", string).replace("__", "_").lower()
+    return _camel_to_snake_case_sub.sub(r"_\1", string).replace("__", "_").lower()
 
 
 def snake_to_camel_case(string: str, capitalize_first: bool = True) -> str:
@@ -1396,6 +1463,19 @@ def parse_json_or_yaml(markup: str) -> JsonComplexType:
                 raise
 
 
+def try_json(data: str):
+    """
+    Tries to deserialize json input to object if possible, otherwise returns original
+    :param data: string
+    :return: deserialize version of input
+    """
+    try:
+        return json.loads(to_str(data or "{}"))
+    except JSONDecodeError:
+        LOG.warning("failed serialize to json, fallback to original")
+        return data
+
+
 def json_safe(item: JsonType) -> JsonType:
     """Return a copy of the given object (e.g., dict) that is safe for JSON dumping"""
     try:
@@ -2163,6 +2243,10 @@ def get_all_subclasses(clazz: Type) -> List[Type]:
     return result
 
 
+def fully_qualified_class_name(klass: Type) -> str:
+    return f"{klass.__module__}.{klass.__name__}"
+
+
 def parallelize(func: Callable, arr: List, size: int = None):
     if not size:
         size = len(arr)
